/*
	binary substitute

	return status:
	0-253	number of substitution occurred
	254		substitution occurred >= 254 times
	255		error
*/

#include <stdio.h>
#include <stdlib.h>
#if defined __MSDOS__ || defined MSDOS
#include <io.h>
#else
#include <unistd.h>
#endif
#include <fcntl.h>
#include <string.h>
#include <assert.h>

#define	MAX_STR_LEN	256
#define	MAX_BUF_LEN	(16*1024)

typedef unsigned char	BYTE;

BYTE	*search_str	(BYTE *buf, int buflen, BYTE *str, int strlen)
{
	int	i;

	assert (buf != NULL);
	assert (buflen > 0);
	assert (str != NULL);
	assert (strlen > 0);
	for (i = 0; i <= buflen - strlen; i++)
	{
		if (memcmp (&buf [i], str, strlen) == 0)
			return &buf [i];
	}
	return NULL;
}

int	hex2bin	(char *str, BYTE *buf)
{
	int	x, i;
	char	*p;

	assert (str != NULL);
	assert (buf != NULL);
	p = strtok (str, " \t\n");
	for (i = 0; p != NULL && i < MAX_STR_LEN; i++)
	{
		sscanf (p, "%x", &x);
		buf [i] = x;
		p = strtok (NULL, " \t\n");
	}
	return i;
}

int	main	(int argc, char **argv)
{
	BYTE	src [MAX_STR_LEN], dst [MAX_STR_LEN];
	BYTE	buf [MAX_BUF_LEN], *p, *q;
	int		srclen, dstlen, occupied, fi, fo, count, result;

	fi = 0;	/* stdin */
	fo = 1;	/* stdout */

	if (argc < 2 || 3 < argc)
	{
		fputs ("usage: bsubst \"srchex\" [\"dsthex\"] <infile >outfile\n", stderr);
		exit (255);
	}

	srclen = hex2bin (argv [1], src);
	if (argc == 2)
		dstlen = 0;
	else
		dstlen = hex2bin (argv [2], dst);
	if (srclen == 0)
		exit (255);

	result = 0;
	occupied = 0;
	while ((count = read (fi, &buf [occupied], MAX_BUF_LEN - occupied)) > 0)
	{
		p = buf;
		while ((q = search_str (p, &buf [occupied + count] - p, src, srclen)) != NULL)
		{
			write (fo, p, q - p);
			write (fo, dst, dstlen);
			p = q + srclen;
			result++;
		}

		if (occupied + count < MAX_BUF_LEN)
		{
			/* EOF or insufficient read from pipe */
			write (fo, p, &buf [occupied + count] - p);
		}
		else
		{
			write (fo, p, &buf [occupied + count - srclen] - p);
			memcpy (buf, &buf [occupied + count - srclen], srclen);
			occupied = srclen;
		}
	}
	exit (result < 254 ? result : 254);
}
