#! /bin/sh
#
# patchaic.sh release 2.10
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/
#
# This script modifies BeOS R4.5.1/4.5.2 aic78xx driver
# for use with some Adaptec SCSI adapters.
# Runnable on BeOS and UNIX to make an installation floppy
#
# 1.00 first release
# 1.10 AHA-2930CU/U supported
# 1.20 R4.5.2 CD driver supported, echo detection bug fixed
# 2.10 Independent of driver version
#
# PCI device ID / vendor ID / subsystem ID / subsystem vendor ID
# AHA-2930CU (AIC-7859): 3860 9004 3869 9004
# AHA-2930U  (AIC-7880): 8878 9004 7888 9004
# AHA-2930U2 (AIC-7890): 0011 9005 0181 9005
# (from FreeBSD 3.x /sys/pci/ahc_pci.c)

[ "x$DIR" = x ] && DIR=/system/add-ons/kernel/busses/scsi
FILE=aic78xx
# R4.5+update1
#OLDSUM451=801274421
# R4.5+update2
#OLDSUM452=786147219
# R4.5.2 CD
#OLDSUM452CD=2930185480
# ???
#OLDSUM=3639127671

if [ ! -x ./bsubst ]
then
	if [ -f bsubst.c ]
	then
		cc -o bsubst bsubst.c
	else
		alert --stop "bsubst is missing."
		exit 1
	fi
fi

# check parameters

if [ $# -ne 1 ]
then
	echo "Usage: $0 {aha2930cu|aha2930u|aha2930u2}"
	exit 1
fi

case $1 in
aha2930cu)
	SRC="04 90 78 86"
	DST="04 90 60 38"
	;;
aha2930u)
	SRC="04 90 78 86"
	DST="04 90 78 88"
	;;
aha2930u2)
	SRC="05 90 20 00"
	DST="05 90 11 00"
	;;
*)
	echo "Unknown chipset: $1"
	exit 1
esac

# check original driver

if [ ! -f $DIR/$FILE ]
then
	echo "Original driver $DIR/$FILE not present."
	exit 1
fi

if [ -f $DIR/$FILE.gz ]
then
	echo "Patch is already applied."
	exit 1
fi

# back up original driver

[ "x$NOBACKUP" = x ] && gzip -c $DIR/$FILE > $DIR/$FILE.gz

# apply patch to driver

< $DIR/$FILE ./bsubst "$SRC" "$DST" > $DIR/$FILE.new
mv $DIR/$FILE.new $DIR/$FILE

echo "All done."
[ "x$NOBACKUP" = x ] && echo "Original driver is in $DIR/$FILE.gz"
exit 0
